import java.awt.Point;
import java.util.BitSet;
import java.util.LinkedList;
import java.util.Queue;
import java.util.Vector;

public class Graph
{
	private double A[][];  			// Adjazenzmatrix
	private int n;         			// Anzahl der Knoten
	private boolean gerichtet; 		// Schalter f. gerichtet
	
	public Graph(int n, boolean gerichtet)
	{
		A        	   = new double[n][n];
		this.n		   = n;
		this.gerichtet = gerichtet;
	}
	public Graph(double A[][], boolean gerichtet)
	{
		this.A         = A;
		this.n         = A.length;
		this.gerichtet = gerichtet;
	}

	// Kante hinzufuegen zwischen (v1, v2) (gerichtet) bzw. {v1, v2} ungerichtet
	public void addKante(int v1, int v2, double gewicht)
	{
		A[v1][v2] = gewicht;
		if (!gerichtet)
			A[v2][v1] = gewicht;	
	}		
	// Kante loeschen zwischen (v1, v2) (gerichtet) bzw. {v1, v2} ungerichtet
	public void loeschenKante(int v1, int v2)
	{
		A[v1][v2] = A[v1][v2] - 1;
		if (!gerichtet)
			A[v2][v1] = A[v2][v1] - 1;	
	}		
	// Rckgabe der Knotenanzahl
	public int getKnotenanzahl()
	{
		return n;
	}
	// Berechnung der Kantenanzahl
	public int getKantenanzahl()
	{
		int m = 0;
		for(int i=0; i<n; i++)
			for(int j=0; j<n; j++)
				if (A[i][j] > 0)
					m = m + 1;
				
		if (gerichtet)
			return m;
		else 
			return m/2;
	}
	// Berechnung des Knotengrad von Knoten v
	public int getKnotengrad(int v)
	{
		int deg = 0;
		for(int i=0; i<n; i++)
			if (A[v][i] > 0)
				deg = deg + 1;
		return deg;
	}
	// Bestimmung des maximalen Knotengrad 
	public int getMaxGrad()
	{
		int max = 0;
		for(int i=0; i<n; i++)
			max = Math.max(max, this.getKnotengrad(i));
		return max;
	}
	// Bestimmung des maximalen Knotengrad 
	public int getMinGrad()
	{
		int min = Integer.MAX_VALUE;
		for(int i=0; i<n; i++)
			min = Math.min(min, this.getKnotengrad(i));
		return min;
	}
	// Bestimme Knoten mit maximalen Knotengrad
	public int getKnotenMaxGrad()
	{
		for(int v=0; v<n; v++)
			if(getKnotengrad(v) == this.getMaxGrad())
				return v;
		return 0;
	}

	// Lschen einer Knotenmenge
	public void loescheKnotenmenge(Vector X)
	{
		double A_neu[][] = new double[n-X.size()][n-X.size()];
		int k1 = 0;
		for(int i=0; i<A.length; i++)
		{
			if (!X.contains(i))
			{
				int k2 = 0;
				for(int j=0; j<A[0].length; j++)
					if (!X.contains(j))
						A_neu[i-k1][j-k2] = A[i][j];
					else
						k2++;
			}
			else
				k1++;
		}
		A = A_neu;	
		n = A_neu.length;
	}
	
	// Ausgabe Matrix A
	public void ausgabeMatrix()
	{
		for(int i=0; i<n; i++)
		{
			for(int j=0; j<n; j++)
			{
				System.out.printf(" %10.1f", A[i][j]);
			}
			System.out.printf("\n");
		}
	}

	// Numerierung der Knoten nach Breitensuche
	public int[] breitensuche(int s)
	{
		// Numerierung der besuchten Knotenreihenfolge
		int zahl[] = new int[n];
			
		// Definition der Warteschlange
		Queue<Integer> Q = new LinkedList<Integer>();
		Q.offer(s);
		
		int v, nr = 1;
		while(Q.size() > 0)
		{
			v = (int) Q.peek();
			if(zahl[v] == 0)     // Knoten noch nicht besucht
			{
				zahl[v] = nr;
				nr      = nr + 1;
			}
			Q.poll(); // Lsche aktuellen Knoten 
			
			// Bestimmung aller Nachbarn
			Vector<Integer> NG = getNachbarn(v); 
			for(int i=0; i<NG.size(); i++)
			{
				int w = (int) NG.elementAt(i);
				if((zahl[w] == 0) && (!Q.contains(w)))  //Test auf kreisfreiheit
					Q.offer(w);
			}
		}		
		return zahl;
	}

	// Numerierung der Knoten nach Tiefensuche
	public void tiefensuche(int s, int zahl[])
	{
		Vector<Integer> NG = getNachbarn(s);
		for(int i=0; i<NG.size(); i++)
		{
			int w = (int) NG.elementAt(i);
			if(zahl[w] == 0)
			{
				zahl[w] = zahl[n];
				zahl[n] = zahl[n] + 1;
				tiefensuche(w, zahl);
			}
		}
	}
		
	// Nachbarn von Knoten v
	public Vector<Integer> getNachbarn(int v)
	{
		Vector NG = new Vector();
		if (v >= 0 && v < n)
		{
			for(int i=0; i<n; i++)
				if (A[v][i] > 0)
					NG.add(i);
					
			return NG;
		}
		else
		{
			System.out.printf("FEHLER: Falsche Knoteneingabe\n");
			return null;
		}
	}
	
	// Index des minimalen Elementes in Vector W mit Werten in d
	public int getMinimum(Vector W, double d[])
	{
		int h, idx = 0;
		double min = Double.MAX_VALUE;
		for(int i=0; i<W.size(); i++)
		{
			h = (int) W.elementAt(i);
			if ((d[h] < min))
			{
				idx = i;
				min = d[h];
			}
		}	
		return idx;
	}
	
	// Index des minimalen Elementes in Vector W mit Werten in d
	public int getMinimum(BitSet W, double d[])
	{
		int h, idx = 0;
		double min = Double.MAX_VALUE;
		for(int i=0; i<d.length; i++)
		{
			if (W.get(i) && d[i] < min)
			{
				idx = i;
				min = d[i];
			}
		}	
		return idx;
	}


	// Index des minimalen Elementes in Vector W_wert
	private int getMinimum(Vector W_wert)
	{
		double h; 
		int idx = 0;
		double min = Double.MAX_VALUE;
		for(int i=0; i<W_wert.size(); i++)
		{
			h = (double) W_wert.elementAt(i);
			if (h < min)
			{
				idx = i;
				min = h;
			}
		}
		
		return idx;
	}

	// Implementierung des Dijkstra Algo fr Krzeste Wege von Knoten s
	public double[] Dijkstra(int s, int vV[])
	{
		// --- 1. Initialisierung der Variablen
		double d[] = new double[n];  	
		Vector<Integer> W = new Vector<Integer>();
		for(int i=0; i<n; i++)
		{
			d[i] = Double.MAX_VALUE;
			W.addElement(i);
		}
		d[s] = 0;
		
		// --- 2. Berechnung des krzesten Weges 
		for(int i=0; i<n; i++)
		{
			// 2a. Berechnung des kleinsten Elementes v in  W
			int v = (int) W.elementAt(getMinimum(W, d));
			W.removeElementAt(getMinimum(W, d));
			
			// 2b. Aktuallisierung der Entfernungen
			Vector<Integer> NG = getNachbarn(v);
			for(int j=0; j<NG.size(); j++)
			{
				int w = (int) NG.elementAt(j);
				if (d[v] + A[v][w] < d[w])
				{
					d[w]  = d[v] + A[v][w];
					vV[w] = v;
				}       	
			}
		}
		return d;
	}
	
	// Berechnung des minimal aufspannenden Baumes
	public Graph berechneAufspannendenBaum()
	{
		// --- 1. Initialisierung der Variablen
		Graph G    = new Graph(n, gerichtet);	
		BitSet Q   = new BitSet();   // Nichtbesuchten Knoten
		double d[] = new double[n];  // Abstand zum Baum T
		int  V[]   = new int[n];     // Vorgaengerknoten
		for(int i=0; i<n; i++)
		{
			Q.set(i);
			d[i]  = Integer.MAX_VALUE;
		}
		d[0] = 0;
		
		// --- 2. Berechnung des Minimalen Spannbaumes
		while(!Q.isEmpty())
		{
			// --- 2a. Bestimme Knoten mit minimalen Abstand
			int u = getMinimum(Q, d);
			for(int v=0; v<n; v++)
			{
				// --- 2b. Aktuallisierung des Abstandes fr alle Nachbarknoten in Q  
				if (A[u][v] > 0 && Q.get(v) && A[u][v] < d[v])
				{
					d[v]  = A[u][v];
					V[v] = u;
				}
			}
			G.addKante(V[u], u, A[V[u]][u]); 
			if (!gerichtet)
				G.addKante(u, V[u], A[V[u]][u]); 			
			Q.clear(u);
		}
		return G;
	}
	
	// Berechnung des maximalen Flusses
	public double berechneMaxFluss(int s, int t)
	{
		// --- 1. Initialisierung des Flusses
		double f[][] = new double[n][n];
		double fmax = 0;
		
		// --- 2. Suche nach Erweiterungspfaden
		while(true)
		{
			// --- 2a. Bestimmung des Graph fr Dijkstra-Algo
			double D[][] = new double[n][n];
			for(int i=0; i<n; i++)
				for(int j=0; j<n; j++)
					if(A[i][j] > 0)
						D[i][j] = 1;
			Graph G = new Graph(D, true);
	
			// --- 2b. Berechnung des Erweiterungspfades
			int vV[] = new int[n];
			G.Dijkstra(s, vV);
			
			// --- 2b. Abbruch: Kein Erweiterungspfad mehr von s nach t
			if (vV[t] == 0)
				break;
			
			// --- 2c. Berechnung des Erweiterungspfades p von s nach t mit Gewicht gew
			Vector p = this.berechnePfad(vV, s); 
			double gew = Double.MAX_VALUE;
			System.out.printf(" Erweiterungspfad: ");
			for(int i=0; i<p.size()-1; i++)
			{
				System.out.printf(" %d ", p.elementAt(i));
				gew = Math.min(gew, A[(int) p.elementAt(i+1)][(int) p.elementAt(i)]);
			}
			
			// --- 2d. Flussverbesserung
		    for(int i=0; i<p.size()-1; i++)
		    {       
		    	int a = (int) p.elementAt(i+1);
		    	int b = (int) p.elementAt(i);
		    	f[a][b] = f[a][b] + gew;  // vorwrts
		    	f[b][a] = f[b][a] - gew;  // rckwrts

		        A[a][b] = A[a][b] - gew;
		    }
		    fmax = fmax + gew;
			System.out.printf(" %d mit Kapazitt = %1.1f und aktuellem Flusswert = %1.1f \n", s, gew, fmax);
			LinAlgebra.ausgabe(f);
		}
		return fmax;
	}
	
	// Berechnung einer Euler-Tour
	public Vector<Integer> berechneEulerTour()
	{
		// --- 1. Initialisierung der Tour
		Vector<Integer> p = new Vector<Integer>();
		int v = 0;
		p.addElement(v);
		
		// --- 2. Bestimmung der Eulertour
		int idx = 1;
		int pos[] = new int[n];
		while(this.getKantenanzahl() > 0)
		{
			// --- 2a. Kantenfolge von v aus durchlaufen
			int i;
			for(i=pos[v]; i<n; i++)
			{
				if(A[v][i] > 0) // Kante vorhanden
				{
					p.add(idx, i);            // Kante hinzufuegen
					pos[v] = i;               // letzte Position setzen
					idx++;                    // Index erhhen
					this.loeschenKante(v, i); // Kante loeschen
					v = i;                    // neuer Startknoten
					break;
				}
			}
			// --- 2b. Neue Kantenfolge mit Starknoten suchen
			if (i==n) 
			{
				for(v=0; v<n; v++)
					if(this.getKnotengrad(v) > 0)
						break;
				idx = p.indexOf(v) + 1;
			}
		}
		return p;
	}
	
	// Bestimmung aller zusammenhngender Komponenten 
	public Vector<Graph> bestimmeKomponenten()
	{
		// --- 1. Anlegen der Datenstruktur aus Graphen
		Vector<Graph> C = new Vector<Graph>(); 
		Graph H = this;	
		BitSet b = new BitSet(); // Knotenstatus 
		for(int i=0; i<H.getKnotenanzahl(); i++)
			b.set(i);
		for(int v=0; v<b.length(); v++)
		{
			// --- 2. Knoten v ist isoliert und frei
			if (H.getKnotengrad(v) == 0 && b.get(v)) 
			{
				Graph G = new Graph(1, false);
				C.add(G);
			}
			// --- 3. Knoten v hat eine oder zwei Kanten und ist frei
			else if (H.getKnotengrad(v) > 0 && b.get(v)) 
			{
				// --- 3a. Bestimmung einer Kantenfolge von Knoten v
				Vector<Integer> p = H.bestimmePfad(v);
				int anz; // Anzahl der Knoten des Pfades
				if (p.firstElement() == p.lastElement()) 
					anz = p.size() - 1; // Kreis
				else
					anz = p.size();     // Weg
				// --- 3b. Graph definieren
				Graph G = new Graph(anz, false);
				for(int i=0; i<p.size()-1; i++)
				{
					G.addKante(p.elementAt(i)%anz, p.elementAt(i+1)%anz, 1);
					b.clear(p.elementAt(i));
					b.clear(p.elementAt(i+1));
				}
				C.add(G);
			}
		}
		return C;
	}
	
	// Bestimmung einer Kantenfolge vom Knoten v
	public Vector<Integer> bestimmePfad(int v)
	{
		Vector<Integer> p = new Vector<Integer>();
		p.add(v);
		do // Solange wie ausgehende Kante vorhanden
		{
			for(int i=0; i<n; i++)
			{
				if (A[v][i] > 0) // Kante (v,i) vorhanden
				{
					p.add(i);
					this.loeschenKante(v, i);
					v = i;
					break;
				}
			}
		}
		while(this.getKnotengrad(v) > 0);
		return p;
	}
	
	// Berechnung der Unabhngigkeitszahl
	public double berechneUnabhngigkeitszahl() 
	{
		 double alpha = 0;
		 if(getMaxGrad() <= 2)
		 {
			 // --- 1. Bestimmung aller zus. Komponenten
			 Vector<Graph> c = bestimmeKomponenten();
			 for(int i=0; i<c.size(); i++)
			 {
				 Graph G = c.elementAt(i);
				 G.ausgabeMatrix();
				 int n   = G.getKnotenanzahl();
				 int m   = G.getKantenanzahl();
 				 if (G.getMaxGrad() == 0) // isolierter Knoten
					 alpha = alpha + 1;
				 else if (m == n-1) // Weg
					 alpha = Math.round(n/2.0);
				 else if (m == n)   // Kreis
					 alpha = n/2;
			 }
		 }
		 else
		 {
			 // --- 1. Auswahl eines Knoten mit maximalen Grad
			 int v = getKnotenMaxGrad();
			 
			 // --- 2. Knoten v ist nicht in der max. unabh. Menge
			 Graph G1 = new Graph(A, gerichtet);
			 Vector<Integer> V1 = new Vector<Integer>();
			 V1.addElement(v);
			 G1.loescheKnotenmenge(V1);    // Lsche v aus G
			 double alpha1 = G1.berechneUnabhngigkeitszahl();          

			 // --- 3. Knoten v ist in der max. unabh. Menge
			 Graph G2 = new Graph(A, gerichtet);
			 Vector<Integer> V2 = this.getNachbarn(v);
			 V2.addElement(v);
			 G2.loescheKnotenmenge(V2);    // Lsche N_G(v) und v aus G
			 double alpha2 = G2.berechneUnabhngigkeitszahl();
		     
			 // --- 4. Bestimmung der Unabhngigkeitszahl
			 alpha = Math.max(alpha1, alpha2 + 1);
		 }
		 return alpha;
	}
	
	// Berechnung des Pfades der Vorgngerknotenmenge vV bis Knoten s 
	public Vector<Integer> berechnePfad(int vV[], int s)
	{
		Vector<Integer> v_pfad = new Vector<Integer>(); 
		int v = n-1;
		while(true)
		{
			if (vV[v] == s)
			{
				v_pfad.add(v);
				v_pfad.add(vV[v]);
				break;
			}
			else
				v_pfad.add(v);
			
			v = vV[v];
		}
		return v_pfad;
	}
	
	// Berechnung einer zulaessingen Faerbung f von Knoten v mit c Farben
	public boolean bestimmeFaerbung(int v, int f[], int c)
	{
		int farbe = 0;
		boolean b;
		do
		{
			farbe = farbe + 1;
			b = false;
			if(pruefeFaerbung(v, f, farbe))
			{
				f[v] = farbe;
				LinAlgebra.ausgabe(f);
				if(v < n - 1)
				{
					b = bestimmeFaerbung(v+1, f, c);
					if(b == false)
						f[v] = 0;
				}
				else
					b = true;
			}
		}
		while((b == false) && (farbe < c));
		return b;
	}
	
	public boolean pruefeFaerbung(int v, int f[], int farbe)
	{
		Vector<Integer> NG = getNachbarn(v);
		for(int i=0; i<NG.size(); i++)
			if(f[NG.elementAt(i)] == farbe)
				return false;
		return true;
	}
	
	// Implementierung des A^*-Algorithmus fr den krzesten s-t-Weg
	public Vector<Integer> Astern(int s, int t, double h[])
	{
		// --- 1. Initialisierung 
		// Pfad von s nach t mit Wegkosten
		Vector<Integer> d = new Vector<Integer>();	
		// Menge W der aktiven Knoten mit Wert f_wert = f_wert_i + A(i,j) + h(j)
		Vector<Integer> W     = getNachbarn(s);
		Vector<Double> f_wert = getfWert(s, 0.0, h); 
		ausgabeVektor(W, f_wert, true);
		
		while(true)
		{
			// --- 2. Bestimmung des Knoten in W mit minimalen f-Wert
			int idx = getMinimum(f_wert);      // Index des Knoten bzgl. W
			int v   = (int) W.elementAt(idx);  
			System.out.printf("  Minimum bei Knoten %d mit Wert %f\n", v, f_wert.elementAt(idx)); 
	
			// --- 3. Abbruchkriterium falls Zielknoten erreicht
			if (v == t)
				break;

			// --- 4. Hinzufgen neuer Nachbarn
			Vector<Integer> NG       = getNachbarn(v);
			Vector<Double> f_NG_wert = getfWert(v, (double) (f_wert.elementAt(idx)) - h[v], h); 
			W.addAll(NG);
			f_wert.addAll(f_NG_wert);
	
			// --- 5. Lschen des ausgewhlten Knotens
			W.removeElementAt(idx);
			f_wert.removeElementAt(idx);
			ausgabeVektor(W, f_wert, true);				
		}
		return d;
	}

	// Nachbarn von Knoten v mit Wert = g(v) + A(v,i) + h(i)
	private Vector<Double> getfWert(int v, double g, double h[])
	{
		Vector<Double> f_wert = new Vector<Double>();
		for(int i=0; i<n; i++)
		{
			if (A[v][i] > 0)
			{
				double wert = g + A[v][i] + h[i];
				f_wert.addElement(wert);
			}
		}
		return f_wert;
	}

	// Ausgabe Vektor
	private void ausgabeVektor(Vector W, Vector W_wert, Boolean b)
	{
		for(int i=0; i<W.size(); i++)
		{
			if (b)
				System.out.printf("%d: %2.1f   ", W.elementAt(i), W_wert.elementAt(i));
			else
				System.out.printf(" %d ", W.elementAt(i));
				
		}
		System.out.printf("\n");
	}
}


