import java.util.Vector;

abstract public class GenetischerAlgorithmus
{
	private int anzInd    = 300;     // Anzahl der Individuen
	private int anzGen    = 5000;    // Anzahl der Generationen
	private double faktor = 0.5;     // Vermischungsfaktor
	private double p      = 0.05;    // Wahrscheinlichkeit der Genmutation	
	private int n;					 // Anzahl der Elemente der Loesung
	
	public void setAnzahlIndividuen(int anzInd)
	{
		this.anzInd = anzInd;
	}
	public void setAnzahlGenerationen(int anzGen)
	{
		this.anzGen = anzGen;
	}
	public void setFaktor(int faktor)
	{
		this.faktor = faktor;
	}
	public void setWhtGenmutation(double p)
	{
		this.p = p;
	}
	protected void setAnzahlLoesungselemente(int n)
	{
		this.n = n;
	}
	
	// Optimierung eines Minimumproblems
	public Loesung optimiere() 	
	{
		System.out.printf("Start der Optimierung: \n");

		Loesung x_opt = null;
		double  w_opt = Integer.MAX_VALUE;
		
		// --- 1. Erzeugung einer Generation von Individuen
		Vector<Loesung> generation = new Vector<Loesung>();
		for(int i=0; i<anzInd; i++)
			generation.add(berechneStartLoesung());
		
		for(int j=0; j<anzGen; j++)
		{
			Vector<Loesung> generation_neu = new Vector<Loesung>();
			for(int i=0; i<anzInd; i++)
			{
				// --- 2. Selektion: Auswahl von zwei Individuen aus der Elterngeneration
				int idx[] = Zufall.randperm(anzInd, 4);
				Loesung x1 = generation.elementAt(idx[0]);
				Loesung x2 = generation.elementAt(idx[1]);
				Loesung x3 = generation.elementAt(idx[2]);
				Loesung x4 = generation.elementAt(idx[3]);
				
				Loesung mutter, vater;
				if(berechneFitnessfunktion(x1) <= berechneFitnessfunktion(x2))
					vater = x1;
				else
					vater = x2;
				if(berechneFitnessfunktion(x3) <= berechneFitnessfunktion(x4))
					mutter = x3;
				else
					mutter = x4;
	
				// --- 3. Kombination: Vermisschung der Gene der Eltern
				int k = (int) Math.round(faktor * n);
				Loesung x = kombiniereLoesung(vater, mutter, k);
				
				// --- 4. Mutation: nderung der Gene mit Wht. p
				x = mutiereLoesung(x, p);  
				generation_neu.add(x);
			
				// --- 5. Bewertung der Loesung
				double w = berechneFitnessfunktion(x);
				if (w < w_opt)
				{
					x_opt = kopiereLoesung(x);
					w_opt = w;
			   		System.out.printf(" Generation %d, Individum %d, %1.4f\n", j, i, berechneFitnessfunktion(x_opt));
				}
		  	}
			for(int i=0; i<anzInd; i++)
			{
				Loesung lsg = kopiereLoesung(generation_neu.elementAt(i));
				generation.setElementAt(lsg, i);
			}
		}
   		System.out.printf(" ---------- %1.4f\n", berechneFitnessfunktion(x_opt));
   		return x_opt;
	}
	
	// Berechnung der Startloesung
	abstract protected Loesung berechneStartLoesung();

	// Berechnung der Fitnessfunktion
	abstract double berechneFitnessfunktion(Loesung lsg);  
	
	// Mutation der Gene der Loesung mit Wht. p 
	abstract Loesung mutiereLoesung(Loesung lsg, double p);  

	// Kompination zweier Loesungen mit Anteil k und 1-k
	abstract Loesung kombiniereLoesung(Loesung lsg1, Loesung lsg2, int k);  

	// Kopieren der aktuellen Loesung
	abstract Loesung kopiereLoesung(Loesung lsg);  
}
