
abstract public class SimulatedAnnealing 
{
	private double c     = 10;	   // Temperaturfaktor
	private int l        = 400;    // Wiederholungsfaktor
	private double eps   = 0.0001; // Abbruchfaktor c > eps 
	private double alpha = 0.90;   // Absenkungsfaktor von c
	
	public void setTemperatur(double c)
	{
		this.c = c;
	}
	public void setWiederholung(int l)
	{
		this.l = l;
	}
	public void setAbbruch(double eps)
	{
		this.eps = eps;
	}
	public void setAbsenkung(double alpha)
	{
		this.alpha = alpha;
	}

	// Optimierung eines Minimumproblems
	public Loesung optimiere() 	
	{
		System.out.printf("Start der Optimierung: \n");
		Loesung x_neu = berechneStartLoesung();
		double w_neu  = berechneZielfunktion(x_neu); 
		Loesung x, x_opt = null; 
		double delta=0, w, w_opt = w_neu;
		int idx = 0;
		while (c > eps)
		{
	    	for(int k=0; k<l; k++)  
	    	{
	        	x     = aendereLoesung(x_neu);
	        	w     = berechneZielfunktion(x);
	        	delta = w - w_neu;

	        	if (w < w_opt)
	        	{
	        		System.out.printf(" %d:  %1.4f\n", idx, Math.abs(w));
	        		x_opt = kopiereLoesung(x);
	        		w_opt = w;
	        	}
	        	if ((delta <= 0) || (Math.exp(-delta/c) > Math.random()))
	        	{
	        		x_neu = kopiereLoesung(x);
	        		w_neu = w;
	        		idx   = idx + 1;
	        	}       	
	    	}  
	    	c = alpha * c;  
		}
		System.out.printf("Ende der Optimierung nach %d Iterationen\n", idx);	
		return x_opt;
	}
	
	// Berechnung der Startloesung
	abstract protected Loesung berechneStartLoesung();

	// Berechnung der Zielfunktion
	abstract double berechneZielfunktion(Loesung lsg);  
	
	// Aenderung der Loesung in der Nachbarschaft
	abstract Loesung aendereLoesung(Loesung lsg);  

	// Kopieren einer Loesung
	abstract Loesung kopiereLoesung(Loesung lsg);  
}
