import java.awt.Color;

public class TestRundreise //Bohrlochproblem
{
	public static void main (String args[])
	{
	    // ----------------- Eingabe -----------------
	    //---1. Raster der Lcher [0, raster] x [0, raster] 
		int raster = 10;
		
		//---2. Anzahl der Lcher
		int n = 30;
				
		// -------------------------------------------
		// --- 1. Bestimmung der Lcherposition
	    int position[][] = erzeugeZufallsBits(raster, n);

	    StdDraw.setXscale(-1, raster);
	    StdDraw.setYscale(-1, raster);
	    for(int i=0; i<n; i++)
	    	StdDraw.filledCircle(position[i][0], position[i][1], 0.3);

		// --- 2. Berechnung der Abstandsmatrix 
		double abstaende[][] = bestimmeAbstand(n, position);
	    		
	    // --- 3. Optimierung der Route
		Rundreiseproblem algo = new Rundreiseproblem(abstaende, position);
		Rundreise lsg         = (Rundreise) algo.optimiere();
		int rundreise[]       = lsg.getRundreise();
		
		// -- 4. Ausgabe der Rundreise
		for(int i=0; i<n+1; i++)
			System.out.printf(" %d", rundreise[i]);
		System.out.printf(" mit Wert = %1.4f.", algo.berechneZielfunktion(lsg));
		zeichnePfad(algo.getPosition(), rundreise);

	}
	
	// Erzeugung von zuflligen und verschiedenen Positionen
	public static int[][] erzeugeZufallsBits(int n, int anz)
	{
		   // 1. Erzeugung Permutation der Zahlen 0,...,n^2-1
		   int perm[] = new int[n*n];
	       for(int i=0; i<n*n; i++)
	           perm[i] = i;

	       // 2. Erzeugung eines Zufallsvektor
	       for(int i=0; i<anz; i++)  
	       {
	           // Zufallszahl zwischen i und n*n-1
	           int r = i + (int) (Math.random() * (n*n-i));
	           // Tausche Elemente von Index i and r
	           int t = perm[r];
	           perm[r] = perm[i];
	           perm[i] = t;
	       }

	       // 3. Umwandlung in n x n
		   int pos[][] = new int[anz][2];
		   for(int i=0; i<anz; i++)
		   {
			   int x = 0;
			   while(perm[i] >= n)
			   {
					perm[i] = perm[i] - n;
					x       = x + 1;
			   }
			   pos[i][0] = x; 
			   pos[i][1] = perm[i];
		   }
	       return pos;  
	}
	
	// Berechnung der Abstandsmatrix aller Knotenpositionen
	public static double[][] bestimmeAbstand(int n, int position[][])
	{
		double M[][] = new double[n][n];	    
	    for (int i=0; i<n; i++)
	    	for(int j=0; j<n; j++)
	    		if (i != j) 
	    			M[i][j] = Math.max(Math.abs(position[i][0]-position[j][0]), Math.abs(position[i][1]-position[j][1]));
	    return M;
	}
	
	public static void zeichnePfad(int pos[][], int x[])
	{
	    StdDraw.setPenColor(Color.red);
		StdDraw.setPenRadius(0.01);
		double sum = 0; 
		int x1, x2, y1, y2, dx, dy;
		for(int i=0; i<x.length-1; i++)
		{
		    StdDraw.setPenColor(Color.white);			
			StdDraw.filledSquare(-1, -1, 0.5);
		    StdDraw.setPenColor(Color.red);
		    x1 = pos[x[i]][0];
		    x2 = pos[x[i+1]][0];
		    y1 = pos[x[i]][1];
		    y2 = pos[x[i+1]][1];
		    dx = x2-x1;
		    dy = y2-y1; 
		    if(Math.abs(dx) >= Math.abs(dy))
		    {
		    	StdDraw.line(x1, y1, x1 + dx/Math.abs(dx)*Math.abs(dy), y2);
		    	StdDraw.line(x1 + dx/Math.abs(dx)*Math.abs(dy), y2, x2, y2);
		    }
		    else 
		    {
		    	StdDraw.line(x1, y1, x2, y1 + dy/Math.abs(dy)*Math.abs(dx));
		    	StdDraw.line(x2, y1 + + dy/Math.abs(dy)*Math.abs(dx), x2, y2);
		    }
			sum = sum + Math.max(Math.abs(dx), Math.abs(dy));
			StdDraw.text(-1, -1, String.valueOf(sum));
			StdDraw.show(300);
		}
	}
}