
public class Automat
{
	private int delta[][];
	private int z0;
	private int F[];
	public Automat(int delta[][], int z0, int F[])
	{
		this.delta = delta;
		this.z0    = z0;
		this.F     = F;
	}
	public int getAnzahlZustaende()
	{
		return delta.length;
	}
	public int getAnzahlAlphabet()
	{
		return delta[0].length;
	}
	public int[][] getUebergangsfunktion()
	{
		return delta;
	}
	public int getAnfangszustand()
	{
		return z0;
	}
	public int[] getEndzustaende()
	{
		return F;
	}

	// --- Loseung des Wortproblems --- 
	public boolean wortproblem(int w[])
	{	
		int z = z0;
		for(int i=0; i<w.length; i++)
		{
			z = delta[z][w[i]];
			if (z==-1)
				return false;
		}
		return this.istEndzustand(z);	
	}

	// berprfung ob Zustand ein Endzustand ist
	public boolean istEndzustand(int z)
	{
		for(int i=0; i<F.length; i++)
			if(z == F[i])
				return true;
		return false;
	}

	// --- Minimierung eines Automaten --- 
	public void minimiereAutomaten()
	{
		int n = this.getAnzahlZustaende();
		
		// --- 1. Aufstellen einer Tabelle mit allen Zustandspaaren {z,z'}
		int Tabelle[][] = new int[n][n];
		
		// --- 2 . Markierung aller Paare {z,z'}, wobei z \in F und z' \notin F
		for (int i=0; i<n; i++) 
			for (int j=0; j<n; j++)
				if (istMarkiert(i, j))
					Tabelle[i][j] = 1;
		LinAlgebra.ausgabe(Tabelle);
		System.out.println();
			
		//--- 3. Fr jedes unmarkierte Paar {z,z'} wird getestet, ob 
		//       {delta(z, a), delta(z', a)} bereits markiert ist. 
		//       Falls ja, wird auch {z,z'} markiert 
		
		boolean adg;
		do
		{
			adg = false;
			for (int i=0; i<n; i++) // fuer alle Tabelleneintraege
			{
				for (int j=0; j<n; j++) // fuer alle Tabelleneintraege
				{
 					if (Tabelle[i][j] == 0) // falls unmarkiertes Paar
 					{
 						for(int a=0; a<this.getAnzahlAlphabet(); a++) // fuer jeden Buchstaben
 						{
							if (Tabelle[delta[i][a]] [delta[j][a]] == 1) // falls Paar markiert
							{
								Tabelle[i][j] = 1;
								adg = true;
							}
 						}
 					}
				}
			}
			LinAlgebra.ausgabe(Tabelle);
			System.out.println();
		}
		while(adg);
		
		// --- 4. Vereinigung aller unmarkierten Paare zu einem Zustand.
		for (int i=1; i<n; i++) 
			for (int j=0; j<i-1; j++)
				if (Tabelle[i][j] == 0)   // unmarkierte Paar
					System.out.printf(" Zustand %d und Zustand %d werden verschmolzen.\n", j, i);	
	}
	
	// berprfung ob Zustandspaar markiert wird
	private boolean istMarkiert(int z1, int z2)
	{
		boolean b1 = istEndzustand(z1);
		boolean b2 = istEndzustand(z2);
		return b1^b2;  // XOR
	}
}