import java.util.Vector;

public class CYK 
{
	int m;     // Anzahl der Nichtterminalsymbole
	int n;     // Anzahl der Terminalsymbole
	AR1 r1[];  // Ableitungsregeln der Form A -> a
	AR2 r2[];  // Ableitungsregeln der Form A -> BC
	
	public CYK(int m, int n, AR1 r1[], AR2 r2[])
	{
		this.m = m;
		this.n = n;
		this.r1 = r1;
		this.r2 = r2;
	}
	
	// Bestimmung der linken Seite aus der Regel A -> a fuer Buchstaben w
	public Vector<Integer> getTerminal(AR1 r[], int w)
	{
		Vector<Integer> v = new Vector<Integer>();
		for(int i=0; i<r.length; i++)
			if(r[i].getRechts() == w)
				v.addElement(r[i].getLinks());
		return v;
	}
	
	// Produkt zweier Mengen von Nichtterminalsymbolen
	public Vector<AR2> vereinige(Vector<Integer> v1, Vector<Integer> v2)
	{
		Vector<AR2> v = new Vector<AR2>(); 
		for(int i=0; i<v1.size(); i++)
			for(int j=0; j<v2.size(); j++)
				v.addElement(new AR2(0, v1.elementAt(i), v2.elementAt(j)));
		return v;
	}
	
	// Bestimmung der linken Seite aus Regel A -> BC fr eine Menge von Nichtterminalsymbolen
	public Vector<Integer> bestimmeAbleitung(Vector<AR2> menge)
	{
		boolean b[] = new boolean[m];
		for(int i=0; i<menge.size(); i++) // alle Ableitungen
		{
			int m[] = menge.elementAt(i).getRechts();
			for(int j=0; j<r2.length; j++)
			{
				int r[] = r2[j].getRechts();
				if(m[0] == r[0] && m[1] == r[1]) 
					b[r2[j].getLinks()] = true;
			}
		}
		
		Vector<Integer> v = new Vector<Integer>();
		for(int i=0; i<b.length; i++)
			if(b[i])
				v.addElement(i);
		
		return v;
	}
	
	// Implementierung des CYK-Algorithmus fr Eingabe w
	public boolean cyk_algo2(int w[])
	{
		int anz = w.length;
		// --- 0. Tabelle fr dynamische Programmierung
		NT tabelle[][] = new NT[anz][anz];
		
		// --- 1. Berechnung der ersten Zeile
		for(int i=0; i<anz; i++)
		{
			tabelle[0][i] = new NT(getTerminal(r1, w[i]));
			ausgabe1(tabelle[0][i].getElement());
		}
		System.out.println();
		
		// --- 2. Berechnung 2. bis n-te Zeile
		for(int i=1; i<anz; i++) // alle Zeilen !!!!!!
		{
			for(int j=0; j<anz-i; j++) // alle Spalten
			{
				Vector<AR2> v_rechts = new Vector<AR2>();
				int id1 = 1+j;  // Index der Tabelle 
				int id2 = i+j+1; 
				//System.out.printf(" [%d, %d] ", id1, id2);
				for(int k=0; k<(id2-id1); k++) // alle Vereinigungsmengen
				{
					//System.out.printf(" +(%d, %d)(%d, %d)+ ", id1, id1+k, id1+k+1, id2); 
					//System.out.printf(" +(%d, %d)(%d, %d)+ ", 0+k, j, i-k-1, j+k+1); 
					v_rechts.addAll(vereinige(tabelle[0+k][j].getElement(), tabelle[i-k-1][j+k+1].getElement()));	
				}
				Vector<Integer> v_links = bestimmeAbleitung(v_rechts);
				tabelle[i][j]           = new NT(v_links);
				ausgabe1(tabelle[i][j].getElement());
			}
			System.out.println();
		}
		
		// --- 3. Prfung nach Element S=0
		if(tabelle[anz-1][0].getElement().indexOf(0)!=-1)
			return true;
		else
			return false;
	}
	
	// Implementierung des CYK-Algorithmus fr Eingabe w
	public boolean cyk_algo(int w[])
	{
		int anz = w.length;
		// --- 0. Tabelle fr dynamische Programmierung
		NT tabelle[][] = new NT[anz][anz];
		
		// --- 1. Berechnung der ersten Zeile
		for(int i=0; i<anz; i++)
			tabelle[0][i] = new NT(getTerminal(r1, w[i]));
		
		// --- 2. Berechnung 2. bis n-te Zeile
		for(int i=1; i<anz; i++) // alle Zeilen !!!!!!
		{
			for(int j=0; j<anz-i; j++) // alle Spalten
			{
				Vector<AR2> v_rechts = new Vector<AR2>();
				int id1 = 1+j;  // Index der Tabelle 
				int id2 = i+j+1; 
	
				for(int k=0; k<(id2-id1); k++) // alle Vereinigungsmengen
					v_rechts.addAll(vereinige(tabelle[0+k][j].getElement(), 
									tabelle[i-k-1][j+k+1].getElement()));	
				Vector<Integer> v_links = bestimmeAbleitung(v_rechts);
				tabelle[i][j]           = new NT(v_links);
			}
		}
		
		// --- 3. Prfung nach Element S=0
		if(tabelle[anz-1][0].getElement().indexOf(0)!=-1)
			return true;
		else
			return false;
	}
	
	
	// ----------- Hilfsfunktionen -----------
	public static void ausgabe1(Vector<Integer> v)
	{
		for(int i=0; i<v.size(); i++)
			System.out.printf(" %d", v.elementAt(i));
		System.out.printf("\t");
	}
		
	public static void ausgabe2(Vector<AR2> v)
	{
		for(int i=0; i<v.size(); i++)
		{
			int h[] = v.elementAt(i).getRechts();
			System.out.printf(" (%d %d) ", h[0], h[1]);
		}
	}	
}
