import java.awt.Color;
import java.util.Stack;
import java.util.Vector;

public class Bereichswachstum implements ISegmentierung
{
	private int saat[];
	private int unten;
	private int oben;
	
	public Bereichswachstum(int p[], int u, int o)
	{
		saat  = p;
		unten = u;
		oben  = o;
	}

	public boolean schwelle(int farbe)
	{
		if (farbe >= unten && farbe <= oben)
			return true;
		else
			return false;
	}

	public void segmentiere(BildSegmentierung bild) 
	{
		int breite  = bild.getBreite()/2;
		int hoehe   = bild.getHoehe();
		// --- 1. Bild kopieren auf rechten Bereich
		for (int x = breite+1; x < 2*breite; x++)  
			for (int y = 0; y < hoehe; y++) 
				bild.set(x, y, bild.get(x-breite, y));		
						
		// --- 2. Initialisierung 
		boolean markiert[][] = new boolean[breite][hoehe];	
		Stack<int[]> aktiv = new Stack<int[]>();
		aktiv.push(saat);
		
		// --- 3. Wachstumsverfahren
		while(aktiv.size() > 0)
		{
			// --- 3a. Oberstes Pixel entnehmen
			int pixel[] = aktiv.pop();
			int x       = pixel[0];
			int y       = pixel[1];	
			if(!markiert[x][y]) // nicht markiert
			{
				markiert[x][y] = true; // Pixel markieren	
				// --- 3b. Prfung ob Homogenittskriterium erfllt
				if(schwelle(bild.get(x, y).getRed()))   
				{
					// --- 3c. Setzung der Farbmarkierung
					bild.set(x+breite, y, new Color(255, 255, 255));		
					// --- 3d. Prfung aller Nachbarn
					Vector<int[]> nachbarn = achtNachbarn(x, y, breite, hoehe);
					for(int i=0; i<nachbarn.size(); i++)
					{
						int p[] = nachbarn.elementAt(i);
						if(!markiert[p[0]][p[1]]) 
							aktiv.push(p);
					}
				}
			}
		}	
	}
	
	
	// Bestimmung der 8-Nachbarschaft
	public Vector<int[]> achtNachbarn(int x, int y, int breite, int hoehe)
	{
		Vector<int[]> nachbarn = new Vector<int[]>();
		if(x > 0 && y > 0)
			nachbarn.add(new int[]{x-1, y-1});
		if(y > 0)
			nachbarn.add(new int[]{x, y-1});
		if(x < breite-1 && y > 0 )
			nachbarn.add(new int[]{x+1, y-1});
		if(x > 0)
			nachbarn.add(new int[]{x-1, y});
		if(x < breite -1)
			nachbarn.add(new int[]{x+1, y});
		if(x > 0 && y < hoehe-1)
			nachbarn.add(new int[]{x-1, y+1});
		if(y < hoehe-1)
			nachbarn.add(new int[]{x, y+1});
		if(x < breite-1 && y < hoehe-1 )
			nachbarn.add(new int[]{x+1, y+1});
		return nachbarn;
	}
}