import java.awt.Color;

public class Glaettung implements IFilter
{
	private String name; 
	private int groesse;
	private double filter[][];
			
	public Glaettung(String name, int groesse)
	{
		this.name    = name;
		this.groesse = groesse;
		this.setFilter();
	}
	
	// Auswahl des Filtertyps
	public void setFilter()
	{
		filter = new double[groesse][groesse]; 
		switch (name)
		{
			case "Mittelwert":
				for(int i=0; i<filter.length; i++)
					for(int j=0; j<filter.length; j++)
						filter[i][j] = berechneMittelwert();
				break;

			case "Gauss":
				double sum = 0;
				for(int i=0; i<filter.length; i++)
				{
					for(int j=0; j<filter.length; j++)
					{
						filter[i][j] = berechneGauss(i - groesse/2, j - groesse/2);
						sum = sum + filter[i][j];
					}
				}
				LinAlgebra.mult(filter, sum);
				break;
			
			default:
				System.out.printf("Filter nicht vorhanden!\n");
		}	
	}
	
	public double berechneMittelwert()
	{
		return 1./(groesse * groesse);
	}
	public double berechneGauss(int x, int y)
	{
		return 1./(2*Math.PI) * Math.exp(-0.5*(x*x + y*y));
	}

	public int conv2(int f[][], int x, int y)
	{
		int m = (groesse - 1)/2;	
		double farbe = 0;
		for (int i=-m; i <= m; i++) 
			for (int j=-m; j <= m; j++) 
				farbe = farbe + f[x+i][y+j] * filter[m+i][m+j];
		return (int) farbe;
	}

	public static int[][] color2Int(BildFilter bild)
	{
		int breite  = bild.getBreite()/2;
		int hoehe   = bild.getHoehe();		
		int bildMat[][] = new int[breite][hoehe];
		for (int x = 0; x < breite; x++) 
			for (int y = 0; y < hoehe; y++) 
				bildMat[x][y] = bild.get(x, y).getRed();  
		return bildMat;
	}
	
	public void filterAlgorithmus(BildFilter bild) 
	{
		int f[][]   = color2Int(bild);
		int breite  = bild.getBreite()/2;
		int hoehe   = bild.getHoehe();
		int m       = (groesse - 1)/2;

		int wert = 0; int max = 255;
		for (int x = breite+1 + m; x < 2*breite-m; x++)  
		{
			for (int y = m; y < hoehe-m; y++) 
			{
				wert = conv2(f, x-breite, y);
				bild.set(x, y, new Color(wert, wert, wert));		
			}
		}
	}
}
