public class Landmarken 
{
	// Aufstellung der Koeffizientenmatrix
	public static double[][] erstelleMatrix(double Px[], double Py[])
	{
		double A[][] = new double[2*Px.length][6];	
		for(int i=0; i<Px.length; i++)
		{
			// Zeile fuer Px[i]
			A[2*i][0]   = Px[i];
			A[2*i][1]   = Py[i];
			A[2*i][4]   = 1;
			// Zeile fuer Py[i]
			A[2*i+1][2] = Px[i];
			A[2*i+1][3] = Py[i];
			A[2*i+1][5] = 1;
		}	
		return A;
	}
	
	// Aufstellung der rechten Seite
	public static double[] erstelleVektor(double Px[], double Py[])
	{
		double b[] = new double[2*Px.length];
		for(int i=0; i<Px.length; i++)
		{
			b[2*i]   = Px[i];
			b[2*i+1] = Py[i];
		}
		return b;
	}
	
	// Anwendung der Methode der kleinsten Quadrate
	public static double[] methKleinsteFehlerQuad(double A[][], double b[])
	{					
		// Aufstellung Matrix B = A^T A
		double B[][] = LinAlgebra.mult(LinAlgebra.transpose(A), A);
		
		// Aufstellung Vektor c = A^T b
		double c[] = LinAlgebra.mult(LinAlgebra.transpose(A), b);
		
		// Loesung Normalgleichung Bx = c
		return LinGS.gs(B, c);
	}
	
	// Berechnung der linearen Transformation y = Ax + b
	public static int[] berechneTransformation(double koeff[], int x1, int x2)
	{
		int y[] = new int[2];
		y[0] = (int) Math.round(koeff[0] * x1 + koeff[1] * x2 + koeff[4]);
		y[1] = (int) Math.round(koeff[2] * x1 + koeff[3] * x2 + koeff[5]);
		return y;
	}
	
	// Transformation des Bildes
	public static void transformiere(Bild bild, double koeff[])
	{
		int breite = bild.getBreite();
		int hoehe  = bild.getHoehe();	
		Bild neu   = new Bild(breite, hoehe);
		boolean markiert[][] = new boolean[breite][hoehe];
		
		// --- 1. Transformation des Bildes
		for(int i=0; i<breite; i++)
		{
			for(int j=0; j<hoehe; j++)
			{
				int x[] = berechneTransformation(koeff, i, j);
				if (x[0] >= 0 && x[1] >= 0 && x[0] < breite && x[1] < hoehe)
				{
					neu.set(x[0], x[1], bild.get(i, j));
					markiert[x[0]][x[1]] = true;
				}
			}
		}
		
		// --- 2. Nachbearbeitung des Bildes bzgl. fehlende Rasterung
		for(int i=1; i<breite-1; i++)
			for(int j=1; j<hoehe-1; j++)
				if(markiert[i][j]==false)
					neu.set(i,j, neu.get(i+1, j));
		neu.anzeigen();
	}
	
	public static void main(String args[])
	{
		// ------------------- Eingabe -------------------
		// --- 1. Bilddaten f. Transformation
		String datei = "MR2.jpg";
		
		//---2. Landmarken von Bild 1 (MR1.jpg)
		double Lx1[] = {20,157,153,178,217,252};
		double Ly1[] = {121,27,254,60,142,151};
		
		//---3. Landmarken von Bild 2 (MR2.jpg)
		double Lx2[] = {34,101,218,136,217,252};
		double Ly2[] = {179,24,226,38,90,79};
				
		// ---------------------------------------------------
		// --- 1. Aufstellung der Matrix A und Vektor b
		double A[][] = erstelleMatrix(Lx2, Ly2);
		double b[]   = erstelleVektor(Lx1, Ly1);
		
		// --- 2. Berechnung der Koeffizienten der lin. Transformation y = Ax+b
		double koeff[] = methKleinsteFehlerQuad(A, b);
		
		// --- 3. Erzeugung des Bildes
		Bild bild = new Bild(datei);
		
		// --- 4. Transfomation des Bildes
		transformiere(bild, koeff);
	}
}

