import java.awt.Color;
import java.util.Vector;

public class LiveWire implements ISegmentierung
{
	public int saat[][];
	public LiveWire(int saat[][])
	{
		this.saat = saat;
	}
	
	public void segmentiere(BildSegmentierung bild) 
	{
		int breite = bild.getBreite()/2;
		int hoehe  = bild.getHoehe();

		// --- 1. Bild kopieren auf rechten Bereich
		for (int x = breite+1; x < 2*breite; x++)  
			for (int y = 0; y < hoehe; y++) 
				bild.set(x, y, bild.get(x-breite, y));		

		// --- 2. Aufstellung des gewichteten Graphen  
		double c[][] = new double[breite][hoehe]; 
		double max   = 0; // maximaler Wert
		for(int x = 1; x < breite; x++)  
		{
			for(int y = 1; y < hoehe; y++) 
			{
				int fx   = bild.get(x, y).getRed();
				int fxdx = bild.get(x-1, y).getRed();
				int fxdy = bild.get(x, y-1).getRed();
				c[x][y]  = Math.sqrt(Math.pow(fx-fxdx, 2) + Math.pow(fx-fxdy, 2)); 
				if(c[x][y] > max)
					max = c[x][y];
			}
		}
		for(int x = 0; x < breite; x++)  
			for(int y = 0; y < hoehe; y++) 
				c[x][y] = 1 - c[x][y]/max;

		for(int i=0; i<saat.length-1; i++)
		{
			// --- 3. Anwendung des Dijkstra-Algorithmus
			int s[] = {saat[i][0],   saat[i][1]};
			int t[] = {saat[i+1][0], saat[i+1][1]};	
			Vector<int[]> v_pfad = Dijkstra(c, s, t);
		
			// --- 4. Einzeichnung des Pfades
			Color farbe = new Color(255, 0, 0);
			for(int j = 0; j < v_pfad.size(); j++)  
			{
				int pixel[] = (int []) v_pfad.elementAt(j);
				bild.set(pixel[0]+breite, pixel[1], farbe);
				bild.set(pixel[0]+breite, pixel[1]+1, farbe);
				bild.set(pixel[0]+breite, pixel[1]-1, farbe);
			}
		}		
	}
	
	// Implementierung des Dijkstra Algo fr Krzeste Wege vom Knoten s zu t
	public static Vector<int[]> Dijkstra(double c[][], int s[], int t[])
	{	
		int breite = c.length;
		int hoehe  = c[0].length;
		int n      = breite * hoehe; 					
			
		// --- 1. Initialisierung der Variablen
		// Lnge der Wege
		double d[] = new double[n];
		// Vorgngerknoten fuer Pfadrekonstruktion
		int vV[] = new int[n];
		// Menge der noch nicht untersuchten Knoten
		Vector<Integer> W = new Vector<Integer>();
		for(int i=0; i<n; i++)
		{
			d[i] = Double.MAX_VALUE;
			if(istBereich(zahl2Punkt(hoehe, i), s, t))
				W.addElement(i);
		}
		d[punkt2Zahl(hoehe, s)] = 0;

		while(W.size() > 0)
		{
			// --- 2. Bestimmung des Minimum in d bzgl. W
			int v = (int) W.elementAt(getMinimum(W, d));
			W.removeElementAt(getMinimum(W, d));
				
			// --- 3. Bestimmung aller Nachbarn von Knoten v
			int vp[] = zahl2Punkt(hoehe, v);
			if (vp[0]>0 && vp[1]>0 && vp[0]<breite-1 && vp[1]<hoehe-1)
			{
				int NG[][] = achtNachbarn(zahl2Punkt(hoehe, v));
				for(int j=0; j<NG.length; j++)
				{
					int w[] = {(int) NG[j][0], (int) NG[j][1]}; 
					if (istBereich(w, s, t))
					{
						// --- 4. Aktuallisierung des Pfades
						int w_zahl = punkt2Zahl(hoehe, w);
						if (d[v] + c[w[0]][w[1]] < d[w_zahl])
						{
							d[w_zahl]  = d[v] + c[w[0]][w[1]];
							vV[w_zahl] = v;
						}    
					}
				}
			}
		}
			
		// --- 5. Berechnung des Pfades
		Vector<int[]> v_pfad = new Vector<int[]>(); 
		int v = punkt2Zahl(hoehe, t);
		while(true)
		{
			if (vV[v] == punkt2Zahl(hoehe, s))
			{
				v_pfad.add(zahl2Punkt(hoehe, v));
				v_pfad.add(zahl2Punkt(hoehe, vV[v]));
				break;
			}
			else
				v_pfad.add(zahl2Punkt(hoehe, v));		
			v = vV[v];
		}		
		return v_pfad;
	}
	
	// Index des Minimalen Elementes in Vector W mit Werten in d
	public static int getMinimum(Vector<Integer> W, double d[])
	{
		int h, idx = -1;
		double min = Double.MAX_VALUE;
		for(int i=0; i<W.size(); i++)
		{
			h = (int) W.elementAt(i);
			if ((d[h] <= min))
			{
				idx = i;
				min = d[h];
			}
		}		
		return idx;
	}

	// Hilfsfunktionen 
	public static int punkt2Zahl(int zeilen, int p[])
	{
		return zeilen * p[1] + p[0];
	}
	public static int[] zahl2Punkt(int zeilen, int zahl)
	{
		int p[] = new int[2];
		p[0] = zahl;
		while(zahl >= zeilen)
		{
			zahl = zahl - zeilen;
			p[1] = p[1] + 1;
			p[0] = p[0] - zeilen;
		}
		return p;
	}
	
	// Bestimmung der 8-Nachbarschaft
	public static int[][] achtNachbarn(int pixel[])
	{
		int nachbarn[][] = new int[8][2];
		int idx = 0;
		for(int dx=-1; dx<=1; dx++)
			for(int dy=-1; dy<=1; dy++)
				if (dx!=0 || dy!=0)
				{
					nachbarn[idx][0] = pixel[0] + dx;
					nachbarn[idx][1] = pixel[1] + dy;
					idx              = idx + 1;
				}
		return nachbarn;
	}
	
	// berprfung ob Pixel p im Rechteck mit den beiden Eckpunkten s und t ist
	public static boolean istBereich(int p[], int s[], int t[])
	{
		int min_x = Math.min(s[0], t[0]);
		int max_x = Math.max(s[0], t[0]);
		int min_y = Math.min(s[1], t[1]);
		int max_y = Math.max(s[1], t[1]);		
		if(p[0] >= min_x && p[0] <= max_x && p[1] >= min_y && p[1] <= max_y)
			return true;
		else 
			return false;
	}
}
